//+------------------------------------------------------------------+
//|                                                   TrailingCT.mqh |
//|                   Copyright 2009-2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Math\Stat\Math.mqh>
#include <Expert\ExpertTrailing.mqh>
#include <ct_11.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Trailing Stop based on 'Category Theory' graph (case 1)    |
//| Type=Trailing                                                    |
//| Name=CategoryTheory                                              |
//| ShortName=CT                                                     |
//| Class=CTrailingCT                                                |
//| Page=trailing_ct                                                 |
//| Parameter=Step,double,0.5,Trailing Step                          |
//| Parameter=LongMap,int,0, Long Map Index                          |
//| Parameter=ShortMap,int,0, Short Map Index                        |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CTrailingCT.                                               |
//| Appointment: Class traling stops with 'Category Theory'          |
//|               graph (case 1).                                    |
//| Derives from class CExpertTrailing.                              |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CTrailingCT : public CExpertTrailing
  {
protected:
   //--- adjusted parameters
   double               m_step;                // trailing step
   
   
   int                           m_long_map;       // Long Map
   int                           m_short_map;       // Short Map

public:
   //--- methods of setting adjustable parameters
   
   
   
   //--- method of verification of settings
   virtual bool         ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool         InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual bool         CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp);
   virtual bool         CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp);
   //---
                        CTrailingCT(void);
                        ~CTrailingCT(void);
   //--- methods of initialization of protected data
   void                 Step(double value)                  { m_step=value;            }
   
   void                 LongMap(int value)                  { m_long_map=value;        }
   void                 ShortMap(int value)                 { m_short_map=value;       }
   

protected:
   
   CGraph<string>       m_graph[2];

   int                  Graph(CGraph<string> &G,int Start=0);
   int                  GraphIndex(int First,int Second);
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CTrailingCT::CTrailingCT(void)
  {
//--- initialization of protected data
      m_used_series=USE_SERIES_TIME+USE_SERIES_SPREAD+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
      
      for(int ii=0;ii<2;ii++)
      {
         m_graph[ii].Arrows(3);
         m_graph[ii].Vertices(4);
         for(int i=0;i<4;i++)
         {
            m_graph[ii].vertex[i].Cardinality(1);
         }
         m_graph[ii].vertex[0].Set(0,"open");
         m_graph[ii].vertex[3].Set(0,"close");
         //
         m_graph[ii].arrow[0].Set(0,0);
         m_graph[ii].arrow[0].Set(1,1);
         m_graph[ii].arrow[1].Set(0,1);
         m_graph[ii].arrow[1].Set(1,2);
         m_graph[ii].arrow[2].Set(0,2);
         m_graph[ii].arrow[2].Set(1,3);
      }
  }
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CTrailingCT::~CTrailingCT(void)
  {
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CTrailingCT::ValidationSettings(void)
  {
//--- validation settings of additional filters
   if(!CExpertTrailing::ValidationSettings())
      return(false);

   if(m_period!=PERIOD_H1){
      printf(__FUNCSIG__+" Timeframe should be one-hour! ");
      return(false);}
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CTrailingCT::InitIndicators(CIndicators *indicators)
  {
//--- check pointer
   if(indicators==NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertTrailing::InitIndicators(indicators))
      return(false);
//--- 
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for long position.          |
//+------------------------------------------------------------------+
bool CTrailingCT::CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
      if(position==NULL)
         return(false);
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      int _x=StartIndex();
      
      int _f=Graph(m_graph[0],0);
      int _s=Graph(m_graph[1],12);
      
      int _g=GraphIndex(_f,_s);
   
      if(!(m_long_map&(1<<_g)))
      {
         return(false);
      }
   
      double _atr=fmax(2.0*m_spread.GetData(_x)*m_symbol.Point(),m_high.GetData(_x)-m_low.GetData(_x));
      
      double _sl=m_low.GetData(_x)-(m_step*_atr);
      
      double level =NormalizeDouble(m_symbol.Bid()-m_symbol.StopsLevel()*m_symbol.Point(),m_symbol.Digits());
      double new_sl=NormalizeDouble(_sl,m_symbol.Digits());
      double pos_sl=position.StopLoss();
      double base  =(pos_sl==0.0) ? position.PriceOpen() : pos_sl;
      
      sl=EMPTY_VALUE;
      tp=EMPTY_VALUE;
      if(new_sl>base && new_sl<level)
         sl=new_sl;
//---
   return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for short position.         |
//+------------------------------------------------------------------+
bool CTrailingCT::CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
      if(position==NULL)
         return(false);
      
      m_high.Refresh(-1);
      m_low.Refresh(-1);
      
      int _x=StartIndex();
      
      int _f=Graph(m_graph[0],0);
      int _s=Graph(m_graph[1],12);
      
      int _g=GraphIndex(_f,_s);
   
      if(!(m_short_map&(1<<_g)))
      {
         return(false);
      }
      
      double _atr=fmax(2.0*m_spread.GetData(_x)*m_symbol.Point(),m_high.GetData(_x)-m_low.GetData(_x));
      
      double _sl=m_high.GetData(_x)+(m_step*_atr);
      
      double level =NormalizeDouble(m_symbol.Ask()+m_symbol.StopsLevel()*m_symbol.Point(),m_symbol.Digits());
      double new_sl=NormalizeDouble(_sl,m_symbol.Digits());
      double pos_sl=position.StopLoss();
      double base  =(pos_sl==0.0) ? position.PriceOpen() : pos_sl;
      
      sl=EMPTY_VALUE;
      tp=EMPTY_VALUE;
      if(new_sl<base && new_sl>level)
         sl=new_sl;
//---
      return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CTrailingCT::Graph(CGraph<string> &G,int Start=0)
   {
      int _graph=-1;
      //
      double _h[],_l[];
      ArraySetAsSeries(_h,true);ArraySetAsSeries(_l,true);
      if(CopyHigh(m_symbol.Name(),m_period,Start,12,_h)>=12 && CopyLow(m_symbol.Name(),m_period,Start,12,_l)>=12)
      {
         int _h_index=ArrayMaximum(_h),_l_index=ArrayMinimum(_l);
         
         if(_h_index<=_l_index)
         {
            G.vertex[1].Set(0,"high");
            G.vertex[2].Set(0,"low");
            _graph=0;
         }
         else if(_h_index>_l_index)
         {
            G.vertex[1].Set(0,"low");
            G.vertex[2].Set(0,"high");
            _graph=1;
         }
      }
      
      return(_graph);
   }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CTrailingCT::GraphIndex(int First,int Second)
   {
      if(First==0 && Second==1)
      {
         return(1);
      }
      else if(First==1 && Second==0)
      {
         return(2);
      }
      else if(First==1 && Second==1)
      {
         return(3);
      }
      
      //
      
      return(0);
   }
//+------------------------------------------------------------------+
